﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace editor
{
    public partial class Form1 : Form
    {
        Scene scene;
        int[,] map;
        int w, h;

        int playerX = 0;
        int playerY = 0;
        int playerLvl = 0;

        const int offset = 60;
        const int cellSize = 50;

        bool LMB = false, LMBp;
        bool RMB = false, RMBp;

        int enemyCount = 0;
        int lvlCount = 2;


        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            scene = new Scene(this.CreateGraphics(), this.Size);

            map = new int[10, 10];  // max
            w = (int)numW.Value;
            h = (int)numH.Value;

            btnClear_Click(null, null);

            lvlCount = (int)numLevels.Value;
            for (int i = 0; i < lvlCount; i++)
                Levels.Add(new Level(1, 1, 3));
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            scene.Resize(this.Size);
        }
        
        private void numSizeChanged(object sender, EventArgs e)
        {
            w = (int)numW.Value;
            h = (int)numH.Value;
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 10; i++)
                for (int j = 0; j < 10; j++)
                    map[i, j] = -1;
        }


        string PassEmptyLine(StreamReader str)
        {
            string s;
            do
            {
                s = str.ReadLine();
            }
            while (String.IsNullOrEmpty(s));
            return s;
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Multiselect = false;
            dlg.Title = "Выберите файл карты";
            dlg.InitialDirectory = Application.StartupPath;

            try
            {
                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    StreamReader str = new StreamReader(dlg.FileName);

                    string[] sSize = PassEmptyLine(str).Split();
                    string[] sData = PassEmptyLine(str).Split();

                    numW.Value = Convert.ToInt32(sSize[0]);
                    numH.Value = Convert.ToInt32(sSize[1]);
                    
                    playerX = Convert.ToInt32(sData[0]);
                    playerY = Convert.ToInt32(sData[1]);
                    playerLvl = Convert.ToInt32(sData[2]);

                    enemyCount = Convert.ToInt32(sData[3]);
                    btnClear_Click(null, null);
                    for (int i = 0; i < enemyCount; i++)
                    {
                        string[] line = PassEmptyLine(str).Split();
                        int x = Convert.ToInt32(line[0]);
                        int y = Convert.ToInt32(line[1]);
                        int s = Convert.ToInt32(line[2]);
                        map[y, x] = s;
                    }

                    Levels.Clear();
                    int levelsCount = Convert.ToInt32(sData[4]);
                    for (int i = 0; i < levelsCount; i++)
                    {
                        string[] line = PassEmptyLine(str).Split();
                        int lvlAdd = Convert.ToInt32(line[0]);
                        int minMobs = Convert.ToInt32(line[1]);
                        int maxMobs = Convert.ToInt32(line[2]);
                        Levels.Add(new Level(lvlAdd, minMobs, maxMobs));
                    }

                    str.Close();
                }
            }
            catch
            {
                MessageBox.Show("Невозможно загрузить карту. Возможно вайл повреждён.");
                numW.Value = 7;
                numH.Value = 7;
                btnClear_Click(null, null);
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Title = "Сохранить файл карты";
            dlg.InitialDirectory = Application.StartupPath;
            dlg.DefaultExt = ".txt";
            dlg.FileName = "map" + (new Random()).Next();

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                StreamWriter stw = new StreamWriter(dlg.FileName);

                stw.WriteLine(w + " " + h);
                stw.WriteLine(String.Format("{0} {1} {2} {3} {4}", playerX, playerY, playerLvl, enemyCount, Levels.levels.Count));
                for (int y = 0; y < h; y++)
                {
                    for (int x = 0; x < w; x++)
                    {
                        if (x == playerX && y == playerY)
                            continue;
                        int cell = map[y, x];
                        if (cell != -1)
                            stw.WriteLine(x + " " + y + " " + cell);
                    }
                }
                foreach (Level lvl in Levels.levels)
                    stw.WriteLine(lvl.lvlAdd + " " + lvl.minMobs + " " + lvl.maxMobs);

                stw.Close();
            }
        }


        private void btnLevels_Click(object sender, EventArgs e)
        {
            (new FormLevels()).ShowDialog();
        }

        void DrawGrid(Graphics g)
        {
            for (int i = 0; i <= h; i++)
            {
                for (int j = 0; j <= w; j++)
                {
                    int y = offset + i * cellSize;
                    g.DrawLine(Pens.Brown, offset, y, offset + w * cellSize, y);
                }
            }
            for (int j = 0; j <= w; j++)
            {
                for (int i = 0; i <= h; i++)
                {
                    int x = offset + j * cellSize;
                    g.DrawLine(Pens.Brown, x, offset, x, offset + h * cellSize);
                }
            }
        }

        private void tmrUpdate_Tick(object sender, EventArgs e)
        {
            Graphics g = scene.GetGraphics();

            g.Clear(Color.WhiteSmoke);

            DrawGrid(g);


            Font font = new Font("Arial", 12);
            Brush mobBrush = new SolidBrush(Color.FromArgb(30, Color.Blue));
            for (int y = 0; y < h; y++)
            {
                for (int x = 0; x < w; x++)
                {
                    int cell = map[y, x];
                    if (cell != -1)
                    {
                        g.FillRectangle(mobBrush, offset + x * cellSize, offset + y * cellSize, cellSize, cellSize);
                        g.DrawString("<" + cell + ">", font, Brushes.Black, offset + x * cellSize, offset + y * cellSize + 20);
                    }
                }
            }


            Pen playerPen = new Pen(Brushes.Coral, 2);
            Brush playerBrush = new SolidBrush(Color.FromArgb(20, Color.Coral));
            g.DrawRectangle(playerPen, offset + playerX * cellSize, offset + playerY * cellSize, cellSize, cellSize);
            g.FillRectangle(playerBrush, offset + playerX * cellSize, offset + playerY * cellSize, cellSize, cellSize);
            g.DrawString("И [" + playerLvl + "]", font, Brushes.Black,
                             offset + playerX * cellSize + 5, offset + playerY * cellSize + 20);


            if (LMB || RMB)
            {
                int mx = Cursor.Position.X - this.Location.X - 9;
                int my = Cursor.Position.Y - this.Location.Y - 30;

                int x = (mx - offset) / cellSize;
                int y = (my - offset) / cellSize;

                if (mx >= offset && my >= offset && x <= w && y <= h)
                {
                    if (LMB && !LMBp)
                    {
                        LMBp = true;
                        if (x == playerX && y == playerY)
                        {
                            playerLvl++;
                        }
                        else
                        {
                            map[y, x]++;
                            if (map[y, x] == 0)
                                enemyCount++;
                        }
                    }
                    if (RMB && !RMBp)
                    {
                        RMBp = true;
                        if (x == playerX && y == playerY)
                        {
                            playerLvl--;
                            if (playerLvl < 0)
                                playerLvl = 0;
                        }
                        else
                        {
                            if (map[y, x] >= 0)
                            {
                                map[y, x]--;
                                if (map[y, x] == -1)
                                    enemyCount--;
                            }
                        }
                    }
                }
            }

            scene.Render();
        }


        private void Form1_MouseDown(object sender, MouseEventArgs e)
        {
            switch(e.Button)
            {
                case MouseButtons.Left:
                    LMB = true;
                    LMBp = false;
                    break;
                case MouseButtons.Right:
                    RMB = true;
                    RMBp = false;
                    break;
                case MouseButtons.Middle:
                    int mx = Cursor.Position.X - this.Location.X - 9;
                    int my = Cursor.Position.Y - this.Location.Y - 30;

                    int x = (mx - offset) / cellSize;
                    int y = (my - offset) / cellSize;

                    if (mx >= offset && my >= offset && x <= w && y <= h)
                    {
                        playerX = x;
                        playerY = y;
                    }
                    break;
            }
        }

        private void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            switch (e.Button)
            {
                case MouseButtons.Left:
                    LMB = false;
                    break;
                case MouseButtons.Right:
                    RMB = false;
                    break;
            }
        }

        private void numLevels_ValueChanged(object sender, EventArgs e)
        {

        }
    }
}
