﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace game
{
    public partial class Form1 : Form
    {
        Scene scene;
        Field field;

        Point logicPos;
        PointF realPos;

        int level;
        int maxLevel;
        int enemyCount;

        const int offset = 40;
        const int cellSize = 60;

        bool motion = false;

        Random random;

        List<Point> variants = new List<Point>();


        public Form1()
        {
            InitializeComponent();

            random = new Random();
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            scene = new Scene(this.CreateGraphics(), this.Size);

            NewGame();
        }


        void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (!motion)
            {
                Point selected = Point.Empty;
                
                switch(e.KeyCode)
                {
                    case Keys.Left:
                        selected = new Point(logicPos.X - 1, logicPos.Y);
                        break;
                    case Keys.Right:
                        selected = new Point(logicPos.X + 1, logicPos.Y);
                        break;
                    case Keys.Up:
                        selected = new Point(logicPos.X, logicPos.Y - 1);
                        break;
                    case Keys.Down:
                        selected = new Point(logicPos.X, logicPos.Y + 1);
                        break;
                }
 
                if (variants.Contains(selected))
                {
                    logicPos = selected;
                    motion = true;
                }
            }
        }


        private void Form1_MouseClick(object sender, MouseEventArgs e)
        {
            if (motion)
                return;

            Point mouse = e.Location;

            if (mouse.X < offset || mouse.Y < offset)
                return;

            int x = (mouse.X - offset) / cellSize;
            int y = (mouse.Y - offset) / cellSize;

            if (x >= field.width || y >= field.height)
                return;

            if (x != logicPos.X && y != logicPos.Y)
                return;

            if (x == logicPos.X && y == logicPos.Y)
                return;

            Point selected = new Point(x, y);
            if (variants.Contains(selected))
            {
                logicPos = selected;
                motion = true;
            }
        }


        private void Form1_Resize(object sender, EventArgs e)
        {
            scene.Resize(this.Size);
        }


        void DrawGrid(Graphics g)
        {
            for (int i = 0; i <= field.height; i++)
            {
                for (int j = 0; j <= field.width; j++)
                {
                    int y = offset + i * cellSize;
                    g.DrawLine(Pens.Brown, offset, y, offset + field.width * cellSize, y);
                }
            }
            for (int j = 0; j <= field.width; j++)
            {
                for (int i = 0; i <= field.height; i++)
                {
                    int x = offset + j * cellSize;
                    g.DrawLine(Pens.Brown, x, offset, x, offset + field.height * cellSize);
                }
            }
        }

        private void tmrUpdate_Tick(object sender, EventArgs e)
        {
            Graphics g = scene.GetGraphics();

            g.Clear(Color.WhiteSmoke);

            DrawGrid(g);


            Font font = new Font("Arial", 12);


            Pen playerPen = new Pen(Brushes.Coral, 2);
            Brush playerBrush = new SolidBrush(Color.FromArgb(20, Color.Coral));
            g.DrawRectangle(playerPen, offset + realPos.X, offset + realPos.Y, cellSize, cellSize);
            g.FillRectangle(playerBrush, offset + realPos.X, offset + realPos.Y, cellSize, cellSize);
            g.DrawString("И [" + level + "]", font, Brushes.Black, offset + realPos.X + 5, offset + realPos.Y + 20);

            Brush mobBrush = new SolidBrush(Color.FromArgb(30, Color.Blue));
            for (int y = 0; y < field.height; y++)
            {
                for (int x = 0; x < field.width; x++)
                {
                    int cell = field[x, y];
                    if (cell != -1)
                    {
                        g.FillRectangle(mobBrush, offset + x * cellSize, offset + y * cellSize, cellSize, cellSize);
                        g.DrawString("<" + cell + ">", font, Brushes.Black, offset + x * cellSize + 5, offset + y * cellSize + 20);
                    }
                }
            }


            Pen varPen = new Pen(Brushes.Green, 2);
            foreach (Point p in variants)
                g.DrawRectangle(varPen, offset + p.X * cellSize, offset + p.Y * cellSize, cellSize, cellSize);


            if (motion)
            {
                int prevDirX = Math.Sign(logicPos.X * cellSize - realPos.X);
                int prevDirY = Math.Sign(logicPos.Y * cellSize - realPos.Y);

                realPos.X += Math.Sign(logicPos.X * cellSize - realPos.X) * (float)tmrUpdate.Interval / 1000 * 600;
                realPos.Y += Math.Sign(logicPos.Y * cellSize - realPos.Y) * (float)tmrUpdate.Interval / 1000 * 600;

                int hoverX = (int)((realPos.X - offset) / cellSize + 1);
                int hoverY = (int)((realPos.Y - offset) / cellSize + 1);
                if ((field[hoverX, hoverY] != -1) && !(hoverX==logicPos.X &&hoverY==logicPos.Y))
                {
                    enemyCount--;
                    if (field[hoverX, hoverY] == level)
                        level += Levels.GetSelected().lvlAdd;
                }
                field[hoverX, hoverY] = -1;

                int dirX = Math.Sign(logicPos.X * cellSize - realPos.X);
                int dirY = Math.Sign(logicPos.Y * cellSize - realPos.Y);
                if (dirX != prevDirX || dirY != prevDirY)
                {
                    realPos = new PointF(logicPos.X * cellSize, logicPos.Y * cellSize);
                    motion = false;

                    GenMobs();

                    RefreshVariants();

                    if (variants.Count == 0)
                    {
                        MessageBox.Show("Вы проиграли!");
                        motion = true;
                    }
                    if (level == maxLevel + 1)
                    {
                        MessageBox.Show("Вы победили!");
                        motion = true;
                    }
                }
            }


            scene.Render();
        }



        Point GetRandomCell()
        {
            List<Point> map_cells = new List<Point>();
            for (int x = 0; x < field.width; x++)
                for (int y = 0; y < field.height; y++)
                    if ((field[x, y] == -1) && !(x == logicPos.X && y == logicPos.Y))
                        map_cells.Add(new Point(x, y));
            return map_cells[random.Next(map_cells.Count)];
        }

        void GenMobs()
        {
            Level level = Levels.GetSelected();
            int count = random.Next(level.minMobs, level.maxMobs + 1);
            for (int i = 0; i < count; i++)
            {
                try
                {
                    Point p = GetRandomCell();
                    enemyCount++;
                    field[p.X, p.Y] = random.Next(maxLevel);
                }
                catch
                {
                    MessageBox.Show("Вы проиграли!");
                    motion = true;
                }
            }
        }

        void RefreshVariants()
        {
            variants.Clear();

            int x, y;
            bool first;

            y = logicPos.Y;
            #region left
            first = true;
            for (x = logicPos.X; x >= 0; x--)
            {
                if (x == logicPos.X)
                    continue;
                if (field[x, y] == -1)
                {
                    if (first)
                        variants.Add(new Point(x, y));
                    break;
                }
                else
                {
                    first = false;
                    if (field[x, y] < level)
                    {
                        variants.Add(new Point(x, y));
                    }
                    else
                        if (field[x, y] == level)
                        {
                            variants.Add(new Point(x, y));
                            break;
                        }
                        else
                            break;
                }
            }
            #endregion
            #region right
            first = true;
            for (x = logicPos.X; x < field.width; x++)
            {
                if (x == logicPos.X)
                    continue;
                if (field[x, y] == -1)
                {
                    if (first)
                        variants.Add(new Point(x, y));
                    break;
                }
                else
                {
                    first = false;
                    if (field[x, y] < level)
                    {
                        variants.Add(new Point(x, y));
                    }
                    else
                        if (field[x, y] == level)
                        {
                            variants.Add(new Point(x, y));
                            break;
                        }
                        else
                            break;
                }
            }
            #endregion

            x = logicPos.X;
            #region up
            first = true;
            for (y = logicPos.Y; y >= 0; y--)
            {
                if (y == logicPos.Y)
                    continue;
                if (field[x, y] == -1)
                {
                    if (first)
                        variants.Add(new Point(x, y));
                    break;
                }
                else
                {
                    first = false;
                    if (field[x, y] < level)
                    {
                        variants.Add(new Point(x, y));
                    }
                    else
                        if (field[x, y] == level)
                        {
                            variants.Add(new Point(x, y));
                            break;
                        }
                        else
                            break;
                }
            }
            #endregion
            #region down
            first = true;
            for (y = logicPos.Y; y < field.height; y++)
            {
                if (y == logicPos.Y)
                    continue;
                if (field[x, y] == -1)
                {
                    if (first)
                        variants.Add(new Point(x, y));
                    break;
                }
                else
                {
                    first = false;
                    if (field[x, y] < level)
                    {
                        variants.Add(new Point(x, y));
                    }
                    else
                        if (field[x, y] == level)
                        {
                            variants.Add(new Point(x, y));
                            break;
                        }
                        else
                            break;
                }
            }
            #endregion
        }


        void NewGame()
        {
            Levels.Clear();
            Levels.Add(new Level(1, 1, 3));
            Levels.Select(0);
            toolLevels.DropDownItems.Clear();
            toolLevels.DropDownItems.Add("1");

            int w = random.Next(5, 11);
            int h = random.Next(5, 11);

            field = new Field(w, h);
            maxLevel = w + h;

            level = 0;

            logicPos = new Point(0, 0);
            realPos = new PointF(0, 0);

            GenMobs();
            RefreshVariants();
        }

        private void btnNewGame_Click(object sender, EventArgs e)
        {
            NewGame();
        }


        string PassEmptyLine(StreamReader str)
        {
            string s;
            do
            {
                s = str.ReadLine();
            }
            while (String.IsNullOrEmpty(s));
            return s;
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Multiselect = false;
            dlg.Title = "Выберите файл карты леса";
            dlg.InitialDirectory = Application.StartupPath;

            try
            {
                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    StreamReader str = new StreamReader(dlg.FileName);

                    string[] sSize = PassEmptyLine(str).Split();
                    string[] sData = PassEmptyLine(str).Split();

                    int w = Convert.ToInt32(sSize[0]);
                    int h = Convert.ToInt32(sSize[1]);
                    field = new Field(w, h);
                    maxLevel = w + h;

                    logicPos.X = Convert.ToInt32(sData[0]);
                    logicPos.Y = Convert.ToInt32(sData[1]);
                    realPos.X = logicPos.X * cellSize;
                    realPos.Y = logicPos.Y * cellSize;
                    level = Convert.ToInt32(sData[2]);

                    enemyCount = Convert.ToInt32(sData[3]);
                    for (int i = 0; i < enemyCount; i++)
                    {
                        string[] line = PassEmptyLine(str).Split();
                        int x = Convert.ToInt32(line[0]);
                        int y = Convert.ToInt32(line[1]);
                        int s = Convert.ToInt32(line[2]);
                        field[x, y] = s;
                    }

                    Levels.Clear();
                    toolLevels.DropDownItems.Clear();
                    int levelsCount = Convert.ToInt32(sData[4]);
                    for (int i = 0; i < levelsCount; i++)
                    {
                        string[] line = PassEmptyLine(str).Split();
                        int lvlAdd = Convert.ToInt32(line[0]);
                        int minMobs = Convert.ToInt32(line[1]);
                        int maxMobs = Convert.ToInt32(line[2]);
                        Levels.Add(new Level(lvlAdd, minMobs, maxMobs));

                        ToolStripItem item = toolLevels.DropDownItems.Add((i + 1).ToString());
                        item.Click += item_Click;
                    }
                    toolLevels.DropDownItems[0].Enabled = false;

                    str.Close();

                    RefreshVariants();
                }
            }
            catch
            {
                MessageBox.Show("Невозможно загрузить карту. Возможно вайл повреждён.");
                NewGame();
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Title = "Сохранить файл карты";
            dlg.InitialDirectory = Application.StartupPath;
            dlg.DefaultExt = ".txt";
            dlg.FileName = "map" + random.Next();

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                StreamWriter stw = new StreamWriter(dlg.FileName);

                stw.WriteLine(field.width + " " + field.height);
                stw.WriteLine(String.Format("{0} {1} {2} {3} {4}", logicPos.X, logicPos.Y, level, enemyCount, Levels.levels.Count));
                for (int y = 0; y < field.height; y++)
                {
                    for (int x = 0; x < field.width; x++)
                    {
                        if (x == logicPos.X && y == logicPos.Y)
                            continue;
                        int cell = field[x, y];
                        if (cell != -1)
                            stw.WriteLine(x + " " + y + " " + cell);
                    }
                }
                foreach (Level lvl in Levels.levels)
                    stw.WriteLine(lvl.lvlAdd + " " + lvl.minMobs + " " + lvl.maxMobs);

                stw.Close();
            }
        }

        void item_Click(object sender, EventArgs e)
        {
            ToolStripItem item = (ToolStripItem)sender;

            foreach (ToolStripItem i in toolLevels.DropDownItems)
                i.Enabled = true;

            Levels.Select(Convert.ToInt32(item.Text) - 1);

            toolLevels.DropDownItems[Levels.selected].Enabled = false;
        }
    }
}
